//+------------------------------------------------------------------+
//|                                       EA Three-in-one system.mq4 |
//|                                         Copyright 2019, Odera FX |
//|                                       mailto:chidera67@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, Odera FX"
#property link      "mailto:chidera67@gmail.com"
#property version   "1.00"
#property strict
#resource "\\Indicators\\Variable Moving Average.ex4"
#include <Trade Functions.mqh>

enum ENUM_TRADE_DIRECTION
  {
   LONG_SHORT, //Long & Short
   LONG_ONLY,  //Long Only
   SHORT_ONLY  //Short Only
   };
   
 enum ENUM_SPECIAL_ORDERS
  {
   SET_PRICE,                         //Set Price
   PRICES_ACROSS_HORIZONTAL_LINE,     //Prices Across Horizontal Line
   PRICES_CROSSES_DIAGONAL_LINE       //Prices Crosses Diagonal Line
  };
  
 enum ENUM_MACD_EXIT_STRATEGY
  {
    DO_NOT_USE_AN_EXIT_SIGNAL,          //Do Not Use An Exit Signal
    MACD_AND_SIGNAL_LINE_CROSS,         //MACD And Signal Line Cross
    HISTOGRAM_CROSSES_ZERO_LINE,       //Histogram Crosses Zero Line
    SIGNAL_LINE_CROSSES_ZERO_LINE     //Signal Line Crosses Zero Line
  };
  
 enum ENUM_VIRTUAL_TP
  {
    PRICE_CROSS_HORIZONTAL_LINE,     //Prices Crosses Horizntal Line
    PRICE_CROSS_DIAGONAL_LINE       //Prices Crosses Diagonal Line
  };
  
    
input string                     hint01                  = "-------------Indicator Settings----------";  //Indicator Settings
input int                        SlowMAPeriod            = 200;                                          //Slow MA Period
input int                        FastMAPeriod            = 100;                                          //Fast MA Period
input ENUM_APPLIED_PRICE         MAPrice                 = PRICE_CLOSE;                                  //MA Price
input ENUM_MA_METHOD             MAMode                  = MODE_SMA;                                     //MA Mode

input int                        FastVMAPeriod           = 10;                                           //Fast VMA Period
input int                        FastVMA_Smoothing       = 4;                                            //Fast VMA Smoothing Period
input int                        SlowVMAPeriod           = 4;                                            //Slow VMA Period
input int                        SlowVMA_Smoothing       = 4;                                            //Slow VMA Smoothing Period

input int                        BBPeriod                = 14;                                           //Bollinger Band Period

input int                        ADXPeriod               = 150;                                          //ADX Period

input int                        RSIPeriod               = 14;                                           //RSI Period

input int                        ROCPeriod               = 9;                                            //Rate of Change Period

input int                        ATRPeriod               = 14;                                           //ATR Period
input double                     ATRMultiplier           = 0.5;                                          //ATR Multiplier

input string                     hint                    = "--------------Standard settings----------";  //Standard Settings
input datetime                   ExpirationDate          = 1000;                                         //Expiration Date
input int                        MagicNumber             = 1000;                                         //Magic Number
input int                        Capital                 = 0;                                            //Capital
input double                     LotSize                 = 1.0;                                          //Lot Size         
input int                        Slippage                = 1;                                            //Slippage
input bool                       TradeOnNewBar           = true;                                         //Trade On New Bar
input bool                       TradeMaxOncePerBar      = false;                                        //Trade Max Once Per Bar
input bool                       InverseStrategy         = false;                                        //Inverse Strategy
input bool                       ReverseOnOppositeSIgnal = false;                                         //Reverse On Opposite Signal
input bool                       CloseOnOppositeSignal   = false;                                        //Close On Opposite Signal(in daily timeframe)
input bool                       AllowHedging            = true;                                         //Allow Hedging

//input string                     hint1                   = "----Commission Per Trade---";                //Commission Per Trade Setup
input bool                       UseCommissionPerTrade   = true;                                         //Use Commission Per Trade
input int                        CommissionPerOrderValue = 0;                                            //Commission Per Order Value (in points)

input bool                       UseFifo                 = true;                                         //Use FIFO
input bool                       UseMultipleOrders       = true;                                         //Use Multiple Orders

input bool                       UseMaxOpenOrders        = true;                                         //Use Max Open Orders
input int                        MaxOpenOrders           = 3;                                            //Max Open Orders

input bool                       UseLossLimitFilter      = true;                                         //Use Loss Limit Filter
input int                        MaxConsecutiveLoses     = 3;                                            //Max Consecutive Loses
input double                     MaxAccumulatedDrawdown  = 10;                                           //Max Accumulated Drawdown(% of capital)

input ENUM_TRADE_DIRECTION       TradeDirection          = LONG_SHORT;                                   //Trade Direction

input bool                       UseSpreadFilter         = false;                                        //Use Spread Filter
input int                        MaxSpread               = 30;                                           //Max Spread (in points)

input bool                       UseMoneyManagement      = false;                                        //Use Money Management
input double                     RiskPercentValue        = 2.0;                                          //Risk Percent Value

input bool                       UseMartingale           = false;                                        //Use Martingale
input int                        MaxMartingaleValue      = 2;                                            //Max Martingale Value
input bool                       ResetAfterMartingale    = true;                                         //Reset After Martingale
input double                     MartingaleValue         = 0.2;                                          //Martingale Value
input bool                       IncreaseOnLoss          = true;                                         //Increase On Loss
input bool                       UseMultiplier           = false;                                        //Use Multiplier

input bool                       UseSpecialOrders        = false;                                        //Use Special Orders
input ENUM_SPECIAL_ORDERS        LongSpecialOrders       = SET_PRICE;                                    //Long Special Orders
input ENUM_SPECIAL_ORDERS        ShortSpecialOrders      = SET_PRICE;                                    //Short Special Orders


input string                     hint1                   = "----------- Exit Strategy ------------";      //Exit Strategy
input bool                       UseAtrTrail             = false;                                         //Use ATR Trail
input int                        AtrPeriodBack           = 14;                                            //ATR Period Back
input double                     AtrMultiplier           = 1.0;                                           //ATR Multiplier
input int                        AtrStep                 = 1;                                             //ATR Step

input bool                       UseMacdCrossoverExit    = false;                                         //Use MACD Crossover Exit
input ENUM_TIMEFRAMES            MacdExitTimeframe       = PERIOD_CURRENT;                                //MACD Exit Timeframe 
input int                        MacdExitFastEmaPeriod   = 12;                                            //MACD Exit Fast EMA Period
input int                        MacdExitSlowEmaPeriod   = 26;                                            //MACD Exit Slow EMA Period
input int                       MacdExitSignalLinePeriod = 9;                                             //MACD Exit Signal Line Period
input ENUM_APPLIED_PRICE         MacdExitAppliedPrice    = PRICE_CLOSE;                                   //MACD Exit Applied Price
input ENUM_MACD_EXIT_STRATEGY    MacdExitStrategy        = DO_NOT_USE_AN_EXIT_SIGNAL;                     //MACD Exit Strategy

input bool                       UseAdxExitSignalFilter  = false;                                         //Use ADX Exit Signal Filter
/*

Configure inputs for adx signal peak

*/

input bool                       UseTakeProfit            = true;                                           //Use Take Profit
//input double                     WhenToTp                 = 0;                                              //When To TP (price, 0 when disabled)
input int                        WhenToTp                 = 300;                                            //When To TP (pips, 0 when disabled)

input bool                       UseVirtualProfitTaking   = false;                                          //Use Virtual Profit Taking

input ENUM_VIRTUAL_TP            VirtualTp                = PRICE_CROSS_HORIZONTAL_LINE;                   //When To TP

input bool                        UseTargetProfit         = false;                                         //Use Target Profit
//input double                      TargetTp                = 0;                                             //Target TP (in price)
input int                         TargetTp                = 0;                                             //Target TP

input bool                        UseProfitTrailingStop   = false;                                         //Use Profit Trailing Stop
input int                         TrailingStop            = 10;                                            //Trailing Stop
input int                         WhenToTrail             = 30;                                            //When To Trail
input int                         TrailingStep            = 1;                                             //Trailing Step

input bool                        UseProfitTrailing       = false;                                         //Use Profit Trailing 3 steps
input int                         ProfitTrailing          = 20;                                            //Trailing Stop, Step 1
input int                         PriceChange             = 25;                                            //Price Change (in pips)
input int                         TimeOfPriceChange       = 2;                                             //Time Of Price Change (in minutes)
//input int                         TrailingStop            = 12;                                          //Trailing Stop, Step 2
input int                         PriceTrail              = 35;                                            //Price Trail
//input int                         TimeOfPriceChange       = 3;                                             //Time Of Price Change

//input int                       TrailingStop            = 8;                                             //Trailing Stop, Step 3
//input int                       PriceChange             = 45;                                            //Price Change (in pips)
//input int                       TimeOfPriceChange       = 5;                                             //Time Of Price Change (in minutes)

input bool                        UseBreakEven            = false;                                         //Use Break Even
input int                         BreakEvenPoint          = 0;                                             //Break Even Point
input int                         PipToLockIn             = 0;                                             //Pip To Lock In

input bool                        UseCandleTrail          = false;                                         //Use Candle Trail
input int                         CandlesBack             = 0;                                             //Candle Back
input int                         PadAmount               = 3;                                             //Pad Amount

input bool                        UseStop                 = true;                                          //Use Stop:Loss
//input double                      WhenToStop              = 0;                                             //When To Stop (price)
input int                         WhenToStop              = 300;                                           //When To Stop(pips)
input bool                        UseStopLossAtrMultiplier= true;                                          //Use Stop Loss ATR Multiplier
input int                         StopLossAtrMultiplier   = 1;                                             //Stop Loss ATR Multiplier

input bool                        UseVirtualStop          = true;                                          //Use Virtual Stop: loss
                        
enum ENUM_MARKET_STATE
  {
   TRENDING_BULL, //Trending upwards
   TRENDING_BEAR, //Trending downwards
   CONSOLIDATING, //Consolidation or sideways movement
   UNDEFINED      //Does not fall into any category
  };

class CRobot
  {
   enum ENUM_INDICATOR
     {
      FAST_VMA,
      SLOW_VMA,
      FAST_MA,
      SLOW_MA,
      SLOPE,
      BB,
      ADX,
      RSI,
      ROC_FAST,
      ROC_SLOW,
      ATR,
      MACD
     };
   
   ENUM_MARKET_STATE GetMarketState();
   
   void              TradingCriteria();
   
   double            GetIndicator(ENUM_INDICATOR name, int shift = 1, int mode = 0);
   double            GetVMA(int period,int smoothing, int shift);
public:
                     CRobot(void){};
                    ~CRobot(void){};
   void              OnTick();
  };

Trade *TF;
CRobot EA;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   
   TF = new Trade(MagicNumber,30,(UseMoneyManagement)?PercentLot:Fixed,LotSize,RiskPercentValue);
//---
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {delete TF;
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   EA.OnTick();
  }
//+------------------------------------------------------------------+
void CRobot::OnTick(void)
{
 if (TF.IsNewBar())
 {
  TradingCriteria();
 }
}
void CRobot::TradingCriteria(void)
{
 if (TF.GetOrderCount() > 0) return;
 switch(GetMarketState())
   {
    case TRENDING_BEAR:{
                        double macd = GetIndicator(MACD,1,MODE_MAIN), macd_prev = GetIndicator(MACD,2,MODE_MAIN),
                               macd_signal = GetIndicator(MACD,1,MODE_SIGNAL), macd_signal_prev = GetIndicator(MACD,2,MODE_SIGNAL);
                        if (macd < macd_signal && macd_prev > macd_signal)
                        {
                         double sl = Bid + (WhenToStop * _Point), tp = Bid - (WhenToTp * _Point);
                         TF.EnterMarketOrder(OP_SELL,Bid,sl,tp,TF.LotGen(OP_SELL,sl),WindowExpertName());
                        }
                       }
                       
      break;
    case TRENDING_BULL:{
                        double macd = GetIndicator(MACD,1,MODE_MAIN), macd_prev = GetIndicator(MACD,2,MODE_MAIN),
                               macd_signal = GetIndicator(MACD,1,MODE_SIGNAL), macd_signal_prev = GetIndicator(MACD,2,MODE_SIGNAL);
                        
                        if (macd > macd_signal && macd_prev < macd_signal)
                        {
                         double sl = Bid - (WhenToStop * _Point), tp = Bid + (WhenToTp * _Point);
                         TF.EnterMarketOrder(OP_BUY,Ask,sl,tp,TF.LotGen(OP_BUY,sl),WindowExpertName());
                        }
                       }
      break;
    case CONSOLIDATING:{
                        double macd = GetIndicator(MACD,1,MODE_MAIN), macd_prev = GetIndicator(MACD,2,MODE_MAIN),
                               macd_signal = GetIndicator(MACD,1,MODE_SIGNAL), macd_signal_prev = GetIndicator(MACD,2,MODE_SIGNAL);
                        
                        if (macd < macd_signal && macd_prev > macd_signal)
                        {
                         double sl = Bid + (WhenToStop * _Point), tp = Bid - (WhenToTp * _Point);
                         TF.EnterMarketOrder(OP_SELL,Bid,sl,tp,TF.LotGen(OP_SELL,sl),WindowExpertName());
                        }
                        else if (macd > macd_signal && macd_prev < macd_signal)
                        {
                         double sl = Bid - (WhenToStop * _Point), tp = Bid + (WhenToTp * _Point);
                         TF.EnterMarketOrder(OP_BUY,Ask,sl,tp,TF.LotGen(OP_BUY,sl),WindowExpertName());
                        }
                       }
     break;
    default:
      break;
   }
}

ENUM_MARKET_STATE CRobot::GetMarketState(void)
{
 //check for uptrend
 double slow_ma = GetIndicator(SLOW_MA);
 if (slow_ma > Close[SlowMAPeriod])
 {
  if (slow_ma > Close[SlowMAPeriod] + (GetIndicator(ATR)*ATRMultiplier))
  {
   double fast_ma = GetIndicator(FAST_MA);
   if (fast_ma > Close[FastMAPeriod])
   {
    if (GetIndicator(SLOPE) > 0)
    {
     if (Close[1] > GetIndicator(BB,1,MODE_UPPER) * 1.5)
     {
      if (GetIndicator(ADX,1,MODE_PLUSDI) > GetIndicator(ADX,1,MODE_MINUSDI))
      {
       if (GetIndicator(RSI) > 53)
       {
        if (GetIndicator(ROC_FAST) > 0 && GetIndicator(ROC_SLOW) > 0)
        {
         if (GetIndicator(FAST_VMA) > GetIndicator(SLOW_VMA))
         {
          Print("Market is trending bullish");
          return TRENDING_BULL;
         }
        }
       }
      }
     }
    }
   }
  }
 }
 
 //check for downtrend
 if (slow_ma < Close[SlowMAPeriod])
 {
  if (slow_ma < Close[SlowMAPeriod] + (GetIndicator(ATR)*ATRMultiplier))
  {
   double fast_ma = GetIndicator(FAST_MA);
   if (fast_ma < Close[FastMAPeriod])
   {
    if (GetIndicator(SLOPE) < 0)
    {
     if (Close[1] < GetIndicator(BB,1,MODE_LOWER) * 1.5)
     {
      if (GetIndicator(ADX,1,MODE_PLUSDI) < GetIndicator(ADX,1,MODE_MINUSDI))
      {
       if (GetIndicator(RSI) < 47)
       {
        if (GetIndicator(ROC_FAST) < 0 && GetIndicator(ROC_SLOW) < 0)
        {
         if (GetIndicator(FAST_VMA) < GetIndicator(SLOW_VMA))
         {
          Print("Market is trending bearish");
          return TRENDING_BEAR;
         }
        }
       }
      }
     }
    }
   }
  }
 }
 //check for consolidating market
 double roc9_6 = ROC(GetVMA(6,4,1),GetVMA(6,4,9)), roc9_12 = ROC(GetVMA(12,4,1),GetVMA(12,4,9)),
        roc9_21 = ROC(GetVMA(21,4,1),GetVMA(21,4,9));
 if (roc9_6 > -0.5 && roc9_6 < 0.5 && roc9_12 > -0.5 && roc9_12 < 0.5 && roc9_21 > -0.5 && roc9_21 < 0.5)
 {
  Print("Market is consolidating");
  return CONSOLIDATING;
 }
 Print("Market does not fall into any defined state");
 return UNDEFINED;
}

double CRobot::GetVMA(int period,int smoothing, int shift)
{
 return iCustom(_Symbol,PERIOD_CURRENT,"::Indicators\\Variable Moving Average.ex4",period,smoothing,0,shift);
}

double CRobot::GetIndicator(ENUM_INDICATOR name,int shift = 1,int mode = 0)
{
 switch(name)
   {
    case FAST_MA: return iMA(_Symbol,PERIOD_CURRENT,FastMAPeriod,0,MAMode,MAPrice,shift);
      break;
    
    case SLOW_MA: return iMA(_Symbol,PERIOD_CURRENT,SlowMAPeriod,0,MAMode,MAPrice,shift);
    
    case SLOW_VMA: return iCustom(_Symbol,PERIOD_CURRENT,"::Indicators\\Variable Moving Average.ex4",SlowVMAPeriod,SlowVMA_Smoothing,0,shift);
    
    case FAST_VMA: return iCustom(_Symbol,PERIOD_CURRENT,"::Indicators\\Variable Moving Average.ex4",FastVMAPeriod,FastVMA_Smoothing,0,shift);
    
    case SLOPE:{
                if (Close[1] - Close[22] > 0 ) return 1;
                else return -1;
               }
    case BB: return iBands(_Symbol,PERIOD_CURRENT,BBPeriod,0,0,PRICE_CLOSE,mode,shift);
    
    case ADX: return iADX(_Symbol,PERIOD_CURRENT,ADXPeriod,PRICE_CLOSE,mode,shift);
    
    case RSI: return iRSI(_Symbol,PERIOD_CURRENT,RSIPeriod,PRICE_CLOSE,shift);
    
    case ROC_SLOW: return ROC(GetIndicator(SLOW_MA,1),GetIndicator(SLOW_MA,ROCPeriod));
    
    case ROC_FAST: return ROC(GetIndicator(FAST_MA,1),GetIndicator(FAST_MA,ROCPeriod));
    
    case ATR: return iATR(_Symbol,PERIOD_CURRENT,ATRPeriod,shift);
    
    case MACD: return iMACD(_Symbol,PERIOD_CURRENT,MacdExitFastEmaPeriod,MacdExitSlowEmaPeriod,MacdExitSignalLinePeriod,MacdExitAppliedPrice,mode,shift);
    default: Print(__FUNCTION__"Undefined Indicator");
    break;
   }
 return EMPTY;
}

double ROC(double currentMA, double pastMA)
{
 return ((currentMA - pastMA)/pastMA)*100;
}